/*global gv_context, gv_utils */

function Particle() {
    this.x = 0;
    this.y = 0;
    this.vx = 0;
    this.vy = 0;
	this.age = 0;
	this.lifespan = 0;
	this.alpha = 1.0;
	this.alphaDelta = 0.0;
	this.scale = 1.0;
    this.scaleDelta = 1.0;
	this.offCenterScale = 1.0;
	this.rotation = 0.0;
	this.rotationSpeed = 0.0;
}

function Emitter(actor) {
    this.actor = actor;
	this.x = 0;
	this.y = 0;

    this.angle = [0, 0];
    this.speed = [0, 0];
    this.alpha = [1, 1];
    this.alphaDelta = [0, 0];
	this.scale = [1, 1];
    this.scaleDelta = [0, 0];
	this.offCenterScale = [1, 1];
	this.rotationSpeed = [0, 0];
    this.lifespan = [1, 1];  // Max Int ??

	this.time =  0;
	this.particles = [];
}

function oneShotUpdate(deltaTimeSeconds) {
    var ii;
    var particle;
    var numParticles = this.particles.length;

    if (this.spawnedAllParticles === undefined) {
        while (this.particles.length < this.spawnRate) {
            this.spawnParticle();
        }
        this.spawnedAllParticles = true;
    }

    for (ii = numParticles - 1; ii >= 0; ii--) {
        particle = this.particles[ii];

        particle.vx += this.gravityX * deltaTimeSeconds;
        particle.vy += this.gravityY * deltaTimeSeconds;

        particle.x += particle.vx * deltaTimeSeconds;
        particle.y += particle.vy * deltaTimeSeconds;

        particle.age += deltaTimeSeconds;
        particle.alpha += particle.alphaDelta * deltaTimeSeconds;
        particle.scale += particle.scaleDelta * deltaTimeSeconds;

        particle.rotation += particle.rotationSpeed * deltaTimeSeconds;

        // remove particles that are no longer visible.
        if (particle.alpha < 0 || particle.age >= particle.lifespan) {
            this.particles.splice(ii, 1);
        }
    }

    if (this.particles.length < 1) {
        if (this.actor && this.actor.killEmitter) {
            this.actor.killEmitter(this);
        }
    }
}

Emitter.prototype.update = function(deltaTimeSeconds) {
    var ii;
    var particle;
    var numParticles = this.particles.length;
    var particlesPerSecond = 1 / this.spawnRate;

    this.time += deltaTimeSeconds;

    while (this.time > particlesPerSecond) {
        this.spawnParticle();
        this.time -=  particlesPerSecond;
    }

	for (ii = numParticles - 1; ii >= 0; ii--) {
		particle = this.particles[ii];

        particle.vx += this.gravityX * deltaTimeSeconds;
        particle.vy += this.gravityY * deltaTimeSeconds;

        particle.x += particle.vx * deltaTimeSeconds;
        particle.y += particle.vy * deltaTimeSeconds;

        particle.age += deltaTimeSeconds;
        particle.alpha += particle.alphaDelta * deltaTimeSeconds;
        particle.scale += particle.scaleDelta * deltaTimeSeconds;

        particle.rotation += particle.rotationSpeed * deltaTimeSeconds;

		// remove particles that are no longer visible.
		if (particle.alpha < 0 || particle.age >= particle.lifespan) {
			this.particles.splice(ii, 1);
		}
	}
};

Emitter.prototype.spawnParticle = function() {
    var angle = gv_utils.getRandomFloat(this.angle[0], this.angle[1]);
    var speed = gv_utils.getRandomFloat(this.speed[0], this.speed[1]);
	var particle = new Particle();

	particle.x = this.x;
	particle.y = this.y;

	particle.vx = Math.cos(angle) * speed;
	particle.vy = Math.sin(angle) * speed;

	particle.lifespan = gv_utils.getRandomFloat(this.lifespan[0], this.lifespan[1]);
	particle.alpha = gv_utils.getRandomFloat(this.alpha[0], this.alpha[1]);
	particle.alphaDelta = gv_utils.getRandomFloat(this.alphaDelta[0], this.alphaDelta[1]);
	particle.scale = gv_utils.getRandomFloat(this.scale[0], this.scale[1]);
    particle.scaleDelta = gv_utils.getRandomFloat(this.scaleDelta[0], this.scaleDelta[1]);
	particle.offCenterScale = gv_utils.getRandomFloat(this.offCenterScale[0], this.offCenterScale[1]);
	particle.rotationSpeed = gv_utils.getRandomFloat(this.rotationSpeed[0], this.rotationSpeed[1]);

	this.particles.push(particle);
};

Emitter.prototype.draw = function() {
    var ii;
    var particle;
    var numParticles = this.particles.length;

	gv_context.save();

    gv_context.globalCompositeOperation = this.compositeMode;

	for (ii = numParticles - 1; ii >= 0; ii--) {
        particle = this.particles[ii];

        gv_context.save();
        gv_context.globalAlpha = particle.alpha;

        gv_context.translate(particle.x, particle.y);
        gv_context.rotate(particle.rotation);
        gv_context.scale(particle.scale, particle.scale);
        gv_context.translate(-particle.x, -particle.y);

        gv_context.drawImage(this.particleImage, particle.x - this.particleImage.width / 2, particle.y - this.particleImage.height / 2, particle.offCenterScale * this.particleImage.width, particle.offCenterScale * this.particleImage.height);

        gv_context.restore();
	}

    gv_context.restore();
};