/*global gv_requestAnimationFrame */

var gv_frameRate = 60;

if (! window.requestAnimationFrame) {
	window.requestAnimationFrame =
		(window.webkitRequestAnimationFrame ||
		window.mozRequestAnimationFrame ||
		window.oRequestAnimationFrame ||
		window.msRequestAnimationFrame ||
		function(callback) {
			return window.setTimeout(callback, 1000 / gv_frameRate);
		});
}

if (! window.cancelAnimationFrame) {
    window.cancelAnimationFrame =
        (window.webkitCancelAnimationFrame ||
        window.mozCancelAnimationFrame ||
        window.oCancelAnimationFrame ||
        window.msCancelAnimationFrame ||
        function() {
            return window.clearTimeout(gv_requestAnimationFrame);
        });
}

var gv_utils = {};

gv_utils.captureMouse = function(element) {
	var mouse = { x : 0, y : 0 };

	element.addEventListener("mousemove", function(event) {
		var x, y;

		if(event.pageX || event.pageY) {
			x = event.pageX;
			y = event.pageY;
		} else {
			x = event.clientX + document.body.scrollLeft + document.documentElement.scrollLeft;
			y = event.clientY + document.body.scrollTop + document.documentElement.scrollTop;
		}

		mouse.x = x - element.offsetLeft;
		mouse.y = y - element.offsetTop;
	}, false);

	return mouse;
};

gv_utils.toRadians = function(degrees) {
	return degrees * Math.PI / 180;
};

gv_utils.toDegrees = function(radians) {
	return radians * 180 / Math.PI;
};

gv_utils.itoa = function(intValue) {
	return String.fromCharCode(intValue);
};

// returns a random integer between min and max (inclusive)
gv_utils.getRandomInt = function(min, max) {
	return Math.floor( Math.random() * (max - min + 1) ) + min;
};

gv_utils.getRandomFloat = function(min, max) {
	return Math.random() * (max - min) + min;
};

// between (x1,y1) to (x2,y2)
gv_utils.getDistance = function(x1, y1, x2, y2) {
	var dx = x2 - x1;
	var dy = y2 - y1;

	return Math.sqrt(dx * dx + dy * dy);
};

// from (x1,y1) to (x2,y2)
gv_utils.getDirection = function(x1, y1, x2, y2) {
	var dx = x2 - x1;
	var dy = y2 - y1;

	var magnitude = Math.sqrt(dx * dx + dy * dy);

	return {
		x : dx / magnitude,
		y : dy / magnitude
	};
};

