/*global gv_gameAudio, gv_utils, loadImage, Emitter, oneShotUpdate */

// Declare global constants
var gv_canvas = document.getElementById("gameCanvas");
var gv_context = gv_canvas.getContext("2d");

var gv_requestAnimationFrame;
var gv_orientationPollerId;
var gv_lastOrientation = {};
var gv_isGameInProgress = false;
var gv_isGameEnding = false;

var gv_images = {};

var gv_previousTime;
var gv_deltaTimeSeconds;

var gv_ship = null;
var gv_numShipEmitters = 5;

var gv_enemies = [];
var gv_numEnemies = 10;
var gv_enemySpeed = 1.0;

var gv_bullets = [];
var gv_numBullets = 10;
var gv_bulletSeparation = 5;  // pixels
var gv_bulletSpeed = 10.0;  // 8.0;
var gv_bulletCreationTimer = null;
var gv_isFiringBullets = false;

// variables for audio
var gv_hurtSound;
var gv_laserSound;
var gv_playerHurtSound;
var gv_isSoundOn = false;

var gv_score = 0;

/*
var gv_playerDifficulties = [
    { difficulty: "easy",   numBullets: 18, numEnemies: 5, enemySpeed: 1.0 },
    { difficulty: "medium", numBullets: 18, numEnemies: 5, enemySpeed: 1.0 },
    { difficulty: "hard",   numBullets: 18, numEnemies: 5, enemySpeed: 1.0 }
]
*/

var gv_playerDifficulties = [
    { difficulty: "easy",   numBullets: 18, numEnemies: 50,  enemySpeed: 1.0 },
    { difficulty: "medium", numBullets: 12, numEnemies: 100, enemySpeed: 2.0 },
    { difficulty: "hard",   numBullets:  6, numEnemies: 150, enemySpeed: 3.0 }
];

function setDifficulty(level) {
    var ii;
    for (ii = 0; ii < gv_playerDifficulties.length; ii++) {
        if (gv_playerDifficulties[ii].difficulty === level) {
            gv_numBullets = gv_playerDifficulties[ii].numBullets;
            gv_numEnemies = gv_playerDifficulties[ii].numEnemies;
            gv_enemySpeed = gv_playerDifficulties[ii].enemySpeed;
            break;
        }
    }
}

/**
 * Log information to the screen.
 *
 * @param text The text to display in the log area.
 */
function log(text) {
    var logArea = document.getElementById("log");
    logArea.innerHTML = text;
}

function reenableControls() {
    var startButton = document.getElementById("start_button");
    var difficultySelector = document.getElementById("difficulty");

    difficultySelector.disabled = false;
    startButton.style.visibility = "visible";
}

function clearScreen() {
    // Draw the background
    gv_context.fillStyle = "#111111";  // "#ECECEC";
    gv_context.fillRect(0, 0, gv_canvas.width, gv_canvas.height);
    gv_context.strokeStyle = "#7F7F7F";
    gv_context.strokeRect(0, 0, gv_canvas.width, gv_canvas.height);
}

function endGame() {
    gv_isGameInProgress = false;
    if (gv_requestAnimationFrame) {
        window.cancelAnimationFrame(gv_requestAnimationFrame);
    }
    if (gv_orientationPollerId) {
        clearInterval(gv_orientationPollerId);
    }
    clearScreen();
    reenableControls();
}

function setGameOver() {
    endGame();
    log("<span class='gameover'>Game Over<\/span>");
}

function setGameWon() {
    endGame();
    log("<span class='gameover'>Congratulations, You Won!<\/span>");
}

/**
 * Process key down events.
 * This is how the ship's position is adjusted, how bullets are fired, and how the game is ended.
 *
 * @param event The key down event.
 */
function onKeyDown(event) {
    if (event.keyCode === 37) {  // left arrow
        gv_ship.isMovingLeft = true;
        event.preventDefault();
    } else if (event.keyCode === 39) {  // right arrow
        gv_ship.isMovingRight = true;
        event.preventDefault();
    } else if (event.keyCode === 32) {  // space key
        gv_isFiringBullets = true;
        event.preventDefault();
    } else if (event.keyCode === 27) {  // escape key
        setGameOver();  // End the game
    }
}

/**
 * Process key up events.
 * This is how the ship's position is adjusted, how bullets are fired, and how the game is ended.
 *
 * @param event The key up event.
 */
function onKeyUp(event) {
    if (event.keyCode === 37) {  // left arrow
        event.preventDefault();
        gv_ship.isMovingLeft = false;
    } else if (event.keyCode === 39) {  // right arrow
        event.preventDefault();
        gv_ship.isMovingRight = false;
    } else if (event.keyCode === 32) {  // space key
        event.preventDefault();
        gv_isFiringBullets = false;
    // } else if (event.keyCode === 27) {  // escape key
        // End the game
    }
}

function fireBulletCallback() {
    if (gv_bulletCreationTimer !== null) {
        gv_bulletCreationTimer = null;
    }
}

function fireBullet() {
    var ii;

    if (gv_isFiringBullets) {
        if (null === gv_bulletCreationTimer) {
            gv_bulletCreationTimer = setTimeout(fireBulletCallback, 150);
            for (ii = 0; ii < gv_bullets.length; ii++) {
                if (! gv_bullets[ii].isAlive) {
                    gv_bullets[ii].x = gv_ship.x;
                    gv_bullets[ii].y = gv_ship.y - gv_ship.h2;
                    gv_bullets[ii].isAlive = true;
                    if (gv_isSoundOn) {
                        gv_laserSound.play();
                    }
                    break;
                }
            }
        }
    } else if (gv_bulletCreationTimer !== null) {
        // The fire button is not pressed - clear the timer.
        clearTimeout(gv_bulletCreationTimer);
        gv_bulletCreationTimer = null;
    }
}

function spawnEnemy() {
    var ii;

    for (ii = 0; ii < gv_enemies.length; ii++) {
        if (! gv_enemies[ii].isSpawned) {
            gv_enemies[ii].getRandomStart();
            gv_enemies[ii].isAlive = true;
            gv_enemies[ii].isSpawned = true;
            setTimeout(spawnEnemy, gv_utils.getRandomInt(50, 500));
            break;
        }
    }
}

function createBulletEmitter(x, y) {
    var emitter = new Emitter();

    emitter.particleImage = gv_images.bullet;
    emitter.compositeMode = "lighter";  // "source-over";
    emitter.x = x || 0;
    emitter.y = y || 0;
    emitter.gravityX = 0;
    emitter.gravityY = 0;
    emitter.spawnRate = 8;
    emitter.time = 1 / emitter.spawnRate;

    emitter.angle = [0, 0];
    emitter.speed = [0, 0];
    emitter.lifespan = [0.6, 1.2];  // for bullet speed = 5: [1.8, 1.8];
    emitter.alpha = [0.8, 0.8];
    emitter.alphaDelta = [-0.1, -0.4];  // for bullet speed = 5: [-0.02, -0.02]
    emitter.scale = [0.4, 0.8];
    emitter.scaleDelta = [0.6, 1.2];  // [1.2, 1.2 ];  // for bullet speed = 5: [+1.0, +1.0];

    return emitter;
}

function createExplosionEmitter(actor, x, y) {
    var emitter = new Emitter(actor);

    emitter.particleImage = gv_images.enemy;
    emitter.compositeMode = "source-over";
    emitter.update = oneShotUpdate;
    emitter.x = x || 0;
    emitter.y = y || 0;
    emitter.gravityX = 0;
    emitter.gravityY = 0;
    emitter.spawnRate = 500;
    emitter.time = 1 / emitter.spawnRate;

    emitter.angle = [0, 2 * Math.PI];
    emitter.speed = [75, 300];
    emitter.lifespan = [0.3, 0.5];
    emitter.alpha = [0.4, 0.8];
    emitter.alphaDelta = [-0.1, -0.4];
    emitter.scale = [0.2, 0.4];
    emitter.scaleDelta = [-0.1, -0.2];

    return emitter;
}

function Actor(image, direction) {
    this.image = image;
    this.direction = direction;
    this.w2 = image.width / 2;  // pre-compute for speed.
    this.h2 = image.height / 2;  // pre-compute for speed.
    this.rSquared = (this.w2 * this.w2) + (this.h2 * this.h2);  // radius squared.
    this.x = this.w2;
    this.y = this.h2;
    this.speed = 0;
    this.angle = 0;
    this.xFactor = 0;
    this.yFactor = 0;
    this.deltaAccel = 0.5;
    this.friction = 1.0;
    this.isMovingLeft = false;
    this.isMovingRight = false;
    this.isAlive = true;
    this.isExploding = false;
    this.globalAlpha = 1.0;
    this.rotation = 0.0;
    // this.scale = 1.0;
    this.scale = 0.3;

    this.computeAngle = function () {
        if (this.direction === "right") {
            this.angle = 0;
        } else if (this.direction === "left") {
            this.angle = Math.PI;
        } else if (this.direction === "up") {
            this.angle = -Math.PI / 2;
        } else {  // "down"
            this.angle = Math.PI / 2;
        }
        this.xFactor = Math.cos(this.angle);  // pre-compute for speed.
        this.yFactor = Math.sin(this.angle);  // pre-compute for speed.
    };
    this.computeAngle();

    this.die = function() {
        this.isAlive = false;
    };

    /**
     * Draw the object at its current location and rotation.
     */
    this.draw = function() {
        if (this.globalAlpha <= 0) {
            return;
        }
        if (this.isExploding) {
            gv_context.save();

            gv_context.globalAlpha = this.globalAlpha * 0.6;
            gv_context.drawImage(this.image, this.x - this.w2, this.y - this.h2);

            gv_context.globalAlpha = this.globalAlpha;
            gv_context.translate(this.x, this.y);
            gv_context.rotate(this.rotation);
            gv_context.scale(this.scale, this.scale);
            gv_context.translate(-this.x, -this.y);
            gv_context.drawImage(gv_images.explosion, this.x - (gv_images.explosion.width / 2), this.y - (gv_images.explosion.height / 2));

            gv_context.restore();

            this.globalAlpha -= 0.015;
            this.rotation += 5;
            this.scale += 0.04;
            if ((this.globalAlpha <= 0) || (this.scale >= 2.2)) {
                this.die();
            }
        } else {
            gv_context.drawImage(this.image, this.x - this.w2, this.y - this.h2);
        }
    };

    this.edgeDetect = function() {
    };

    this.updateSpeed = function() {
    };

    /**
     * Update the actor's speed, rotation, and position.
     */
    this.update = function() {
        if (! this.isAlive) {
            return;
        }

        this.updateSpeed();

        this.speed *= this.friction;

        if ((this.speed > -0.005) && (this.speed < 0.005)) {
            this.speed = 0.0;  // Ensure braking to zero.
        }

        if (! gv_isGameEnding) {
            this.x += this.speed * this.xFactor;
            this.y += this.speed * this.yFactor;
            this.edgeDetect();
        }

        this.draw();
    };

    this.hitTest = function(object2) {
        var dx = object2.x - this.x;  // X-distance between objects
        var dy = object2.y - this.y;  // Y-distance between objects
        var rSquared = (dx * dx) + (dy * dy);  // Square of distance between objects
        // Is square of distance between objects <= sum of object radii squared?  (Yes == hit).
        return (rSquared <= (this.rSquared + object2.rSquared));
    };

    this.explode = function() {
        this.isExploding = true;
        this.friction = 0;  // stop all movement.
    };

}  // end: Actor

function createShip() {
    var ship;

    ship = new Actor(gv_images.ship, "right");
    ship.friction = 0.90;
    ship.emitters = null;

    ship.x = gv_canvas.width / 2;
    ship.y = gv_canvas.height - 80;

    ship.edgeDetect = function() {
        if (this.x - this.w2 <= 0) {
            this.x = this.w2;
            this.direction = "right";
            this.computeAngle();
        } else if (this.x + this.w2 >= gv_canvas.width) {
            this.x = gv_canvas.width - this.w2;
            this.direction = "left";
            this.computeAngle();
        }
    };

    ship.updateSpeed = function() {
        var ii;
        if (this.isMovingRight) {
            if (this.direction === "right") {
                this.speed += this.deltaAccel;
            } else if (this.direction === "left") {
                this.speed -= this.deltaAccel;
            }
        } else if (this.isMovingLeft) {
            if (this.direction === "right") {
                this.speed -= this.deltaAccel;
            } else if (this.direction === "left") {
                this.speed += this.deltaAccel;
            }
        }
        if (this.emitters && (this.emitters.length > 0)) {
            for (ii = 0; ii < this.emitters.length; ii++) {
                this.emitters[ii].update(gv_deltaTimeSeconds);
                this.emitters[ii].draw();
            }
        }
    };

    /**
     * Draw the object at its current location and rotation.
     */
    ship.draw = function() {
        var ii;
        var dx;
        var dy;
        var dd;

        if (this.globalAlpha <= 0) {
            return;
        }
        if (this.isExploding) {
            // Draw the ship
            gv_context.save();
            gv_context.globalAlpha = this.globalAlpha * 0.6;
            gv_context.drawImage(this.image, this.x - this.w2, this.y - this.h2);
            gv_context.restore();

            // Draw the explosion rings
            dd = Math.min(this.w2, this.h2) * this.scale;
            for (ii = 0; ii < gv_numShipEmitters; ii++) {
                dx = 0;
                dy = 0;
                if (ii > 0) {
                    if ((ii % 2) === 0) {
                        dx = -dd;  // -this.w2;
                    } else {
                        dx = dd;  // this.w2;
                    }
                    if (ii < 3) {
                        dy = -dd;  // -this.h2;
                    } else {
                        dy = dd;  // this.h2;
                    }
                }
                gv_context.save();
                gv_context.globalAlpha = this.globalAlpha;
                gv_context.translate(this.x + dx, this.y + dy);
                gv_context.rotate(this.rotation);
                gv_context.scale(this.scale, this.scale);
                gv_context.drawImage(gv_images.explosion,  -(gv_images.explosion.width / 2), -(gv_images.explosion.height / 2));
                gv_context.restore();
            }

            this.globalAlpha -= 0.01;
            this.rotation += 5;
            this.scale += 0.02;
            if (this.globalAlpha <= 0) {
                this.die();
            }
        } else {
            gv_context.drawImage(this.image, this.x - this.w2, this.y - this.h2);
        }
    };

    ship.killEmitter = function(emitter) {
        var ii;
        if (this.emitters) {
            for (ii = 0; ii < this.emitters.length; ii++) {
                if (this.emitters[ii] === emitter) {
                    this.emitters[ii].particles = [];
                    this.emitters[ii] = null;
                    this.emitters.splice(ii, 1);
                }
            }
            if (this.emitters.length <= 0) {
                this.isAlive = false;
            }
        }
    };

    ship.die = function() {
        var ii;
        var dx;
        var dy;
        var dd;
        if (! this.emitters) {
            ship.emitters = [];
            dd = Math.min(this.w2, this.h2) * this.scale;
            for (ii = 0; ii < gv_numShipEmitters; ii++) {
                dx = 0;
                dy = 0;
                if (ii > 0) {
                    if ((ii % 2) === 0) {
                        dx = -dd;  // -this.w2;
                    } else {
                        dx = dd;  // this.w2;
                    }
                    if (ii < 3) {
                        dy = -dd;  // -this.h2;
                    } else {
                        dy = dd;  // this.h2;
                    }
                }
                this.emitters[ii] = createExplosionEmitter(this, this.x + dx, this.y + dy);
            }
        }
    };

    return ship;
}

function createBullet() {
    var bullet;

    bullet = new Actor(gv_images.bullet, "up");
    bullet.speed = gv_bulletSpeed;
    bullet.isAlive = false;
    bullet.leftEmitter = createBulletEmitter();
    bullet.rightEmitter = createBulletEmitter();

    bullet.die = function() {
        this.isAlive = false;
        this.leftEmitter.particles = [];
        this.rightEmitter.particles = [];
    };

    bullet.edgeDetect = function() {
        if (this.y <= 0) {
            this.die();
        }
    };

    bullet.hitTest = function(object2) {
        var dy;
        var dySquared;
        var dxLeft;
        var dxRight;
        var rSquaredLeft;
        var rSquaredRight;

        dy = object2.y - this.y;  // Y-distance between objects
        dySquared = dy * dy;
        dxLeft = object2.x - this.x - gv_bulletSeparation;  // X-distance between objects (left bullet)
        dxRight = object2.x - this.x + gv_bulletSeparation;  // X-distance between objects (right bullet)
        rSquaredLeft = (dxLeft * dxLeft) + dySquared;  // Square of distance between object and left bullet
        rSquaredRight = (dxRight * dxRight) + dySquared;  // Square of distance between object and right bullet
        // Is square of distance between objects <= sum of object radii squared?  (Yes == hit).
        return ((rSquaredLeft <= (this.rSquared + object2.rSquared)) ||
            (rSquaredRight <= (this.rSquared + object2.rSquared)));
    };

    bullet.draw = function() {
        gv_context.drawImage(this.image, this.x - gv_bulletSeparation - this.w2, this.y - this.h2);
        gv_context.drawImage(this.image, this.x + gv_bulletSeparation - this.w2, this.y - this.h2);
    };

    bullet.updateSpeed = function() {
        this.leftEmitter.x = this.x - gv_bulletSeparation;
        this.leftEmitter.y = this.y;
        // this.leftEmitter.update(gv_deltaTimeSeconds);
        this.leftEmitter.update(1.0);  // for bullet speed = 5: 0.5
        this.leftEmitter.draw();

        this.rightEmitter.x = this.x + gv_bulletSeparation;
        this.rightEmitter.y = this.y;
        // this.rightEmitter.update(gv_deltaTimeSeconds);
        this.rightEmitter.update(1.0);  // for bullet speed = 5: 0.5
        this.rightEmitter.draw();
    };

    return bullet;
}

function createEnemy() {
    var enemy;

    enemy = new Actor(gv_images.enemy, "down");
    enemy.speed = gv_enemySpeed;
    enemy.isSpawned = false;
    enemy.isAlive = false;
    enemy.emitter = null;


    enemy.getRandomStart = function() {
        this.x = gv_utils.getRandomInt(0, gv_canvas.width);
        this.y = - this.h2 - this.h2;
    };

    enemy.edgeDetect = function() {
        if (this.y >= gv_canvas.height) {
            this.getRandomStart();
        }
    };

    enemy.killEmitter = function(emitter) {
        if (this.emitter && (this.emitter === emitter)) {
            this.isAlive = false;
            this.emitter.particles = [];
            this.emitter = null;
        }
    };

    enemy.updateSpeed = function() {
        if (this.emitter) {
            this.emitter.update(gv_deltaTimeSeconds);
            this.emitter.draw();
        }
    };

    enemy.die = function() {
        if (! this.emitter) {
            this.emitter = createExplosionEmitter(this, this.x, this.y);
        }
    };

    return enemy;
}

function hitTestShipByEnemies() {
    var ii;

    for (ii = 0; ii < gv_enemies.length; ii++) {
        if (gv_enemies[ii].isAlive) {
            if (gv_ship.hitTest(gv_enemies[ii])) {
                gv_enemies[ii].explode();
                if (gv_isSoundOn) {
                    gv_hurtSound.play();
                }
                gv_isGameEnding = true;
                gv_ship.explode();
                if (gv_isSoundOn) {
                    gv_playerHurtSound.play();
                }
            }
        }
    }
}

function hitTestEnemiesByBullets() {
    var ii;
    var jj;

    for (ii = 0; ii < gv_bullets.length; ii++) {
        if (gv_bullets[ii].isAlive) {
            for (jj = 0; jj < gv_enemies.length; jj++) {
                if (gv_enemies[jj].isExploding) {
                    continue;
                }
                if (gv_enemies[jj].isAlive) {
                    if (gv_bullets[ii].hitTest(gv_enemies[jj])) {
                        gv_bullets[ii].die();
                        gv_enemies[jj].explode();
                        if (gv_isSoundOn) {
                            gv_hurtSound.play();
                        }
                        gv_score += 10;
                        break;
                    }
                }
            }
        }
    }
}

function displayScore() {
    // var text = "Score: " + gv_score + " dead: " + getNumEnemiesDead() + " active: " + getNumEnemiesActive() + " total: " + gv_numEnemies;
    var text = "Score: " + gv_score;
    var textSize;

    gv_context.save();
    gv_context.strokeStyle = "#FFFFFF";
    gv_context.font = "30px Arial";
    textSize = gv_context.measureText(text);
    gv_context.strokeText(text, (gv_canvas.width - textSize.width) / 2, gv_canvas.height - 20);
    gv_context.restore();
}

function getNumEnemiesActive() {
    var ii;
    var numEnemiesActive = 0;

    for (ii = 0; ii < gv_enemies.length; ii++) {
        if ((! gv_enemies[ii].isExploding) && (gv_enemies[ii].isAlive)) {
            numEnemiesActive++;
        }
    }
    return numEnemiesActive;
}

function getNumEnemiesDead() {
    var ii;
    var numEnemiesDead = 0;

    for (ii = 0; ii < gv_enemies.length; ii++) {
        if (gv_enemies[ii].isExploding && (! gv_enemies[ii].isAlive)) {
            numEnemiesDead++;
        }
    }
    return numEnemiesDead;
}

function checkIfGameWon() {
    var ii;
    var numEnemiesDead = 0;

    for (ii = 0; ii < gv_enemies.length; ii++) {
        if (gv_enemies[ii].isExploding && (! gv_enemies[ii].isAlive)) {
            numEnemiesDead++;
        }
    }
    if ((gv_enemies.length > 0) && (numEnemiesDead >= gv_numEnemies)) {
        setGameWon();
    }
}

function updateBullets() {
    var ii;

    if (gv_isFiringBullets && (null === gv_bulletCreationTimer)) {
        fireBullet();
    }

    for (ii = 0; ii < gv_bullets.length; ii++) {
        if (gv_bullets[ii].isAlive) {
            gv_bullets[ii].update();
        }
    }
}

function updateEnemies() {
    var ii;

    for (ii = 0; ii < gv_enemies.length; ii++) {
        if (gv_enemies[ii].isAlive) {
            gv_enemies[ii].update();
        }
    }
}

function doUpdateGame() {
    var currentTime = new Date();
    gv_deltaTimeSeconds = (currentTime - gv_previousTime) / 1000;

    clearScreen();

    gv_ship.update();

    updateBullets();
    updateEnemies();

    hitTestShipByEnemies();

    if (! gv_ship.isAlive) {
        setGameOver();
    }

    hitTestEnemiesByBullets();

    displayScore();
    checkIfGameWon();

    gv_previousTime = currentTime;
}

/**
 * This function is called once every update of the animation frame.
 * It draws the background, updates the object positions, draws the objects,
 * and displays the score.
 */
function updateGame() {
    if (! gv_isGameInProgress) {
        return;
    }
    doUpdateGame();
}

function initShip() {
    gv_ship = createShip();
}

function initBullets() {
    var ii;

    gv_bullets = [];

    for (ii = 0; ii < gv_numBullets; ii++) {
        gv_bullets.push(createBullet());
    }
}

function initEnemies() {
    var ii;

    gv_enemies = [];

    for (ii = 0; ii < gv_numEnemies; ii++) {
        gv_enemies.push(createEnemy());
    }
}

function onDeviceOrientationUpdate(event) {
    var xDelta;
    // var yDelta;

    if ((! gv_isGameInProgress) || (! gv_ship.isAlive)) {
        return;
    }

    switch (window.orientation) {
        case 0:
            xDelta = gv_lastOrientation.gamma;
            // yDelta = gv_lastOrientation.beta;
            break;
        case 180:
            xDelta = gv_lastOrientation.gamma * -1;
            // yDelta = gv_lastOrientation.beta * -1;
            break;
        case 90:
            xDelta = gv_lastOrientation.beta;
            // yDelta = gv_lastOrientation.gamma * -1;
            break;
        case -90:
            xDelta = gv_lastOrientation.beta * -1;
            // yDelta = gv_lastOrientation.gamma;
            break;
        default:
            xDelta = gv_lastOrientation.gamma;
            // yDelta = gv_lastOrientation.beta;
    }

    gv_ship.deltaAccel = 0.2;  // Slow down for tablets.
    gv_ship.isMovingLeft = false;
    gv_ship.isMovingRight = false;
    xDelta = Math.floor(xDelta / 8);
    if (xDelta > gv_ship.deltaAccel) {
        gv_ship.isMovingRight = true;
    } else if (xDelta < -gv_ship.deltaAccel) {
        gv_ship.isMovingLeft = true;
    }
    // log("ship speed: " + gv_ship.speed.toFixed(5) + ", xDelta: " + xDelta.toFixed(5) + ", yDelta: " + yDelta.toFixed(5));
    log("ship speed: " + gv_ship.speed.toFixed(4) + ", xDelta: " + xDelta.toFixed(4));
}

function onDeviceOrientationChange(event) {
    gv_lastOrientation.gamma = event.gamma;
    gv_lastOrientation.beta = event.beta;
}

function onTouchStart(event) {
    event.preventDefault();
    gv_isFiringBullets = true;
}

function onTouchEnd(event) {
    gv_isFiringBullets = false;
}

function onTouchCancel(event) {
    gv_isFiringBullets = false;
}

// Does the gyroscope or accelerometer actually work?
function deviceOrientationTest(event) {
    window.removeEventListener('deviceorientation', deviceOrientationTest);
    if ((event.beta !== null) && (event.gamma !== null)) {
        window.addEventListener('deviceorientation', onDeviceOrientationChange, false);
        gv_canvas.addEventListener('touchstart', onTouchStart, false);
        gv_canvas.addEventListener('touchend', onTouchEnd, false);
        gv_canvas.addEventListener('touchcancel', onTouchCancel, false);
        gv_orientationPollerId = setInterval(onDeviceOrientationUpdate, 10);
    }
}

function startGame() {
    gv_previousTime = new Date();

    gv_isGameEnding = false;
    gv_isGameInProgress = true;

    initShip();
    initBullets();
    initEnemies();
    spawnEnemy();
    log("");  // Clear the log text

    if (gv_orientationPollerId) {
        gv_orientationPollerId = setInterval(onDeviceOrientationUpdate, 10);
    }

    (function drawFrame() {
        gv_requestAnimationFrame = window.requestAnimationFrame(drawFrame, gv_canvas);
        updateGame();
    }());
}

function initSounds() {
    gv_hurtSound = gv_gameAudio.createSound("hurt");
    gv_hurtSound.play();
    gv_hurtSound.pause();

    gv_laserSound = gv_gameAudio.createSound("laser");
    gv_laserSound.play();
    gv_laserSound.pause();

    gv_playerHurtSound = gv_gameAudio.createSound("playerHurt");
    gv_playerHurtSound.play();
    gv_playerHurtSound.pause();
}

function sndInit(){
    initSounds();
    document.getElementById('appleMobileSoundInit').style.display = 'none';
}

function appleMobileSoundInit() {
    if ((navigator.userAgent.match(/iPhone/i)) || (navigator.userAgent.match(/iPod/i)) || (navigator.userAgent.match(/iPad/i))) {
        document.write('<a id="appleMobileSoundInit" ontouchstart="javascript:sndInit();"></a>');
    } else {
        initSounds();
    }
}

function addListeners() {
    var startButton = document.getElementById("start_button");
    var soundOnRadioButton = document.getElementById("sound_on");
    var soundOffRadioButton = document.getElementById("sound_off");
    var difficultySelector = document.getElementById("difficulty");

    window.addEventListener("keydown", onKeyDown, false);
    window.addEventListener("keyup", onKeyUp, false);
    window.addEventListener('deviceorientation', deviceOrientationTest, false);

    startButton.onclick = function() {
        gv_isSoundOn = soundOnRadioButton.checked;
        setDifficulty(difficultySelector.value);
        difficultySelector.disabled = true;
        this.style.visibility = "hidden";
        startGame();
    };
    difficultySelector.onchange = function() {
        setDifficulty(this.value);
    };
    soundOnRadioButton.onclick = function() {
        gv_isSoundOn = this.checked;
        if (gv_laserSound === undefined) {
            initSounds();
        }
    };
    soundOffRadioButton.onclick = function() {
        gv_isSoundOn = (! this.checked);
    };

    soundOnRadioButton.checked = gv_isSoundOn;
    startButton.disabled = false;
}

/**
 * Initialize the game.
 */
function initGame() {
    gv_ship = createShip();
    addListeners();
    doUpdateGame();  // Show ship in starting location.
}

appleMobileSoundInit();

// Load the images.  This game is started after the last image completes loading.
loadImage("bullet");
loadImage("explosion");
loadImage("enemy");
loadImage("ship");
